import logging
from random import randrange

from pyplanet.apps.config import AppConfig
from pyplanet.contrib.command import Command
from pyplanet.utils.style import style_strip
from pyplanet.apps.core.maniaplanet.models import Player

from .bakery import Muffin, roll_muffin
from .models import PlayerMuffin

logger = logging.getLogger(__name__)

class MuffinsApp(AppConfig):
	game_dependencies = []
	app_dependencies = ['core.maniaplanet', 'core.trackmania', 'core.shootmania']


	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)


	async def on_start(self):
		await self.instance.permission_manager.register('baking', 'Allowed to bake muffins in the bakery', app=self, min_level=2, namespace='muffin')
		await self.instance.command_manager.register(
			Command(command='muffin', aliases=[], target=self.bake_muffin, admin=True, perms='muffin:baking', description='Bake a muffin for a player')
				.add_param(name='player', nargs='*', type=str, required=True),
			Command(command='eat', aliases=[], target=self.eat_muffin, namespace='muffin', description='Eat one of your muffins... yummy!'),
			Command(command='brag', aliases=[], target=self.brag_muffin, namespace='muffin', description='Show off one of your rarest muffins. Guaranteed to make everyone jealous!'),
			Command(command='give', aliases=[], target=self.give_muffin, namespace='muffin', description='Give one of your muffins to another player. How magnanimous of you!')
				.add_param(name='player', nargs='*', type=str, required=True),
		)


	async def bake_muffin(self, player: Player, data, **kwargs) -> None:
		data_player = ' '.join(data.player)
		target_player = await self._lookup_player(data_player)
		if not target_player:
			await self.instance.chat(f'$f00No player found for $<$fff{data_player}$>', player)
			return
		muffin = roll_muffin()
		muffin_text = await self._get_muffin_text(muffin)
		await self.instance.chat(f'$ff0$<$fff{player.nickname}$> » {muffin_text} for $<$fff{target_player.nickname}$>')
		await self.instance.chat(f'$0cfYou received a muffin! Use $<$fff/muffin eat$> to eat it, $<$fff/muffin brag$> to show off, or $<$fff/muffin give$> to give it to another player', target_player.login)

		logger.debug('Create new entry for muffin')
		try:
			await PlayerMuffin.execute(PlayerMuffin.insert(login=target_player.login, muffin_name=muffin.name, muffin_tier=int(muffin.tier)))
		except Exception as e:
			logger.error(e)


	async def give_muffin(self, player: Player, data, **kwargs) -> None:
		data_player: str = ' '.join(data.player) if data.player else ''
		target_player = await self._lookup_player(data_player)
		if not target_player:
			await self.instance.chat(f'$f00No player found for $<$fff{data_player}$>', player)
			return

		player_muffins = await self._get_muffins(player.login)
		if len(player_muffins) > 0:
			min_tier = min([int(muffin.muffin_tier) for muffin in player_muffins])
			min_tier_muffins = [muffin for muffin in player_muffins if int(muffin.muffin_tier) == min_tier]
			player_muffin = min_tier_muffins[randrange(0, len(min_tier_muffins))]
			muffin = Muffin.from_playermuffin(player_muffin)
			muffin_text = await self._get_muffin_text(muffin)
			await self.instance.chat(f'$ff0$<$fff{player.nickname}$> gives their $<$fff{muffin_text}$> to $<$fff{target_player.nickname}$>')
			await self.instance.chat(f'$0cfYou received a muffin! Use $<$fff/muffin eat$> to eat it, $<$fff/muffin brag$> to show off, or $<$fff/muffin give$> to give it to another player', target_player.login)

			logger.debug('Updating the login of muffin with id ' + str(player_muffin.id))
			try:
				await PlayerMuffin.execute(PlayerMuffin.update(login=target_player.login).where(PlayerMuffin.id == player_muffin.id))
			except Exception as e:
				logger.error(e)
		else:
			await self.instance.chat(f'$f00You don\'t have any muffins to give', player)


	async def eat_muffin(self, player: Player, data, **kwargs) -> None:
		players_muffins = await self._get_muffins(player.login)
		if len(players_muffins) > 0:
			player_muffin = players_muffins[randrange(0, len(players_muffins))]
			muffin = Muffin.from_playermuffin(player_muffin)
			muffin_text = await self._get_muffin_text(muffin)
			await self.instance.chat(f'$ff0$<$fff{player.nickname}$> eats their $<$fff{muffin_text}$>. It tastes {muffin.get_taste()}')

			logger.debug('Deleting muffin with id ' + str(player_muffin.id))
			try:
				await PlayerMuffin.execute(PlayerMuffin.delete().where(PlayerMuffin.id == player_muffin.id))
			except Exception as e:
				logger.error(e)
		else:
			await self.instance.chat('$f00You don\'t have any muffins to eat', player)


	async def brag_muffin(self, player: Player, data, **kwargs) -> None:
		player_muffins = await self._get_muffins(player.login)
		if len(player_muffins) > 0:
			max_tier = max([int(muffin.muffin_tier) for muffin in player_muffins])
			max_tier_muffins = [muffin for muffin in player_muffins if int(muffin.muffin_tier) == max_tier]
			display_muffin = max_tier_muffins[randrange(0, len(max_tier_muffins))]
			muffin_text = await self._get_muffin_text(Muffin.from_playermuffin(display_muffin))
			await self.instance.chat(f'$ff0$<$fff{player.nickname}$> shows off their $<$fff{muffin_text}$>. They have {str(len(player_muffins))} muffin(s)')
		else:
			await self.instance.chat('$f00You don\'t have any muffins', player)


	async def _get_muffins(self, login) -> 'list[PlayerMuffin]':
		login_muffins = await PlayerMuffin.execute(PlayerMuffin.select().where(PlayerMuffin.login == login))
		return list(login_muffins) if len(login_muffins) > 0 else []


	async def _lookup_player(self, search_text) -> Player:
		online_players = self.instance.player_manager.online # type: list[Player]
		for online_player in online_players:
			if online_player.login == search_text:
				logger.debug('Found player by login ' + search_text)
				return online_player
		for online_player in online_players:
			if style_strip(online_player.nickname).lower() == search_text.lower():
				logger.debug('Found player by nickname ' + search_text)
				return online_player
		return None


	async def _get_muffin_text(self, muffin) -> str:
		muffin_text = f'$<$n$fff{muffin.fmt_tier()}$> ' if muffin.fmt_tier() else ''
		muffin_text += f'$<$fff{muffin.name}$>'
		return muffin_text



from peewee import *
from pyplanet.core.db import TimedModel


class PlayerMuffin(TimedModel):
	login = CharField(null=False, max_length=150)
	"""
	Player login that the muffin belongs to
	"""

	muffin_name = CharField(null=False, max_length=150)
	"""
	The name of the muffin
	"""

	muffin_tier = IntegerField(null=False)
	"""
	The tier level of the muffin
	"""

	class Meta:
		db_table = 'muffins_playermuffin'



import logging
import random
from enum import IntEnum

logger = logging.getLogger(__name__)


class MuffinTier(IntEnum):
	NORMAL = 100
	UNCOMMON = 200
	RARE = 300
	LEGENDARY = 400

	@staticmethod
	def fmt_string(value: 'MuffinTier') -> str:
		if value == MuffinTier.NORMAL:
			return ''
		elif value == MuffinTier.UNCOMMON:
			return '$3bb[Uncommon]'
		elif value == MuffinTier.RARE:
			return '$73e[Rare]'
		elif value == MuffinTier.LEGENDARY:
			return '$ff9[L$fe9e$ee9g$ed9en$ec9d$dc9a$db9ry]'
		else:
			return str(value)


class Muffin:
	def __init__(self, name: str, tier: MuffinTier) -> None:
		self.name = name
		self.tier = tier

	def fmt_tier(self) -> str:
		return MuffinTier.fmt_string(self.tier)

	def get_taste(self) -> str:
		tastes = []
		if self.tier == MuffinTier.LEGENDARY:
			tastes += [
				'heavenly!',
				'scrumdiddlyumptious!',
				'sublime!',
				'amazing!',
				'incredible!',
				'awesome!',
				'fabulous!',
				'perfect!',
				'outstanding!',
				'stupendous!',
				'wonderful!',
				'fantastic!',
				'divine!',
			]
		elif self.tier == MuffinTier.RARE:
			tastes += [
				'flavorful.',
				'zesty.',
				'unique.',
				'quite yummy.',
				'quite good.',
				'savory.',
				'excellent.',
				'remarkable.',
				'delectable.',
				'delightful.',
				'succulent.',
				'rich.',
				'scrumptious.',
				'piquant.',
			]
		elif self.tier == MuffinTier.UNCOMMON:
			tastes += [
				'good.',
				'pretty good.',
				'nice.',
				'yummy.',
				'pleasant.',
			]
		else:
			tastes += [
				'okay.',
				'alright.',
				'interesting.',
				'palatable.',
			]
		return random.choice(tastes)

	@staticmethod
	def from_playermuffin(player_muffin) -> 'Muffin':
		return Muffin(player_muffin.muffin_name, MuffinTier(player_muffin.muffin_tier))


def _create_muffin_list() -> 'list[Muffin]':
	return [
		Muffin('Muffin', MuffinTier.NORMAL),

		Muffin('Blueberry Muffin', MuffinTier.UNCOMMON),
		Muffin('Chocolate Muffin', MuffinTier.UNCOMMON),
		Muffin('Pancake Muffin', MuffinTier.UNCOMMON),
		Muffin('Cream Cheese Muffin', MuffinTier.UNCOMMON),
		Muffin('Pumpkin Muffin', MuffinTier.UNCOMMON),
		Muffin('Banana Muffin', MuffinTier.UNCOMMON),
		Muffin('Chocolate Chip Muffin', MuffinTier.UNCOMMON),
		Muffin('Cinnamon Muffin', MuffinTier.UNCOMMON),
		Muffin('Mini Cherry Muffin', MuffinTier.UNCOMMON),
		Muffin('Zucchini Muffin', MuffinTier.UNCOMMON),
		Muffin('Strawberry Muffin', MuffinTier.UNCOMMON),
		Muffin('Applesauce Muffin', MuffinTier.UNCOMMON),
		Muffin('Cranberry Muffin', MuffinTier.UNCOMMON),
		Muffin('Poppy Seed Muffin', MuffinTier.UNCOMMON),
		Muffin('English Muffin', MuffinTier.UNCOMMON),
		Muffin('Quiche Muffin', MuffinTier.UNCOMMON),
		Muffin('Honey Muffin', MuffinTier.UNCOMMON),
		Muffin('Monkey Muffin', MuffinTier.UNCOMMON),
		Muffin('Brown Sugar Muffin', MuffinTier.UNCOMMON),
		Muffin('Carrot Muffin', MuffinTier.UNCOMMON),
		Muffin('Cornbread Muffin', MuffinTier.UNCOMMON),
		Muffin('Avocado Muffin', MuffinTier.UNCOMMON),

		Muffin('Banana Blueberry Muffin', MuffinTier.RARE),
		Muffin('Crumb Apple Muffin', MuffinTier.RARE),
		Muffin('Lemon Blueberry Muffin', MuffinTier.RARE),
		Muffin('Lemon Crumb Muffin', MuffinTier.RARE),
		Muffin('Banana Bread Muffin', MuffinTier.RARE),
		Muffin('Skinny Chocolate Muffin', MuffinTier.RARE),
		Muffin('Fruity Pebbles Muffin', MuffinTier.RARE),
		Muffin('Mini Crumb-Cake Muffin', MuffinTier.RARE),
		Muffin('Zucchini Cream Cheese Muffin', MuffinTier.RARE),
		Muffin('Pumpkin Brownie Muffin', MuffinTier.RARE),
		Muffin('Pumpkin Cream Cheese Muffin', MuffinTier.RARE),
		Muffin('Pumpkin Streusel Muffin', MuffinTier.RARE),
		Muffin('Bacon Pepper and Egg Muffin', MuffinTier.RARE),
		Muffin('Blueberry Monkey Bread Muffin', MuffinTier.RARE),
		Muffin('Lemon-Raspberry Streusel Muffin', MuffinTier.RARE),
		Muffin('Maple-Chai Pumpkin Muffin', MuffinTier.RARE),
		Muffin('Avocado Pineapple Muffin', MuffinTier.RARE),
		Muffin('Sweet Potato Muffin', MuffinTier.RARE),
		Muffin('Banana Oat Muffin', MuffinTier.RARE),
		Muffin('Blueberry Cream Muffin', MuffinTier.RARE),
		Muffin('Peanut Butter Banana Muffin', MuffinTier.RARE),
		Muffin('Sweet Corn Muffin', MuffinTier.RARE),
		Muffin('Feta and Chive Muffin', MuffinTier.RARE),
		Muffin('Carmel Apple Muffin', MuffinTier.RARE),
		Muffin('Blueberry Orange Muffin', MuffinTier.RARE),
		Muffin('Coconut Chocolate Muffin', MuffinTier.RARE),
		Muffin('Buttermilk Cranberry Muffin', MuffinTier.RARE),
		Muffin('Pumpkin Surprise Muffin', MuffinTier.RARE),
		Muffin('Brown Sugar Oat Muffin', MuffinTier.RARE),
		Muffin('Morning Maple Muffin', MuffinTier.RARE),
		Muffin('Lemon Meringue Muffin', MuffinTier.RARE),
		Muffin('Breakfast Egg Muffin', MuffinTier.RARE),
		Muffin('Cinnamon Doughnut Muffin', MuffinTier.RARE),
		Muffin('Rhubarb Streusel Muffin', MuffinTier.RARE),
		Muffin('Whole Wheat Blueberry Muffin', MuffinTier.RARE),
		Muffin('Pineapple Upside-Down Muffin', MuffinTier.RARE),
		Muffin('Irish Soda Bread Muffin', MuffinTier.RARE),
		Muffin('Pumpkin Banana Muffin', MuffinTier.RARE),
		Muffin('Wild Blueberry Muffin', MuffinTier.RARE),
		Muffin('Sausage Pancake Muffin', MuffinTier.RARE),
		Muffin('Coconut Carrot Muffin', MuffinTier.RARE),
		Muffin('Frozen Blueberry Muffin', MuffinTier.RARE),
		Muffin('Coffee Cake Muffin', MuffinTier.RARE),
		Muffin('Apple Streusel Muffin', MuffinTier.RARE),
		Muffin('Cranberry Pumpkin Muffin', MuffinTier.RARE),
		Muffin('Sour Cream Chip Muffin', MuffinTier.RARE),
		Muffin('Cranberry Sweet Potato Muffin', MuffinTier.RARE),
		Muffin('Zucchini Chocolate Chip Muffin', MuffinTier.RARE),
		Muffin('Chocolate Chip Oatmeal Muffin', MuffinTier.RARE),
		Muffin('Walnut Zucchini Muffin', MuffinTier.RARE),
		Muffin('Apple Spice Muffin', MuffinTier.RARE),
		Muffin('Pecan Pie Mini Muffin', MuffinTier.RARE),
		Muffin('Sausage Cheese Muffin', MuffinTier.RARE),
		Muffin('Ginger Pear Muffin', MuffinTier.RARE),
		Muffin('Banana Macadamia Muffin', MuffinTier.RARE),
		Muffin('Pumpkin Spice Muffin', MuffinTier.RARE),
		Muffin('Cranberry Nut Muffin', MuffinTier.RARE),
		Muffin('Cheddar Cheese Muffin', MuffinTier.RARE),
		Muffin('Pepperoni Pizza Muffin', MuffinTier.RARE),
		Muffin('Oatmeal Raisin Muffin', MuffinTier.RARE),
		Muffin('Mango Coconut Muffin', MuffinTier.RARE),

		Muffin('Chocolate Zucchini Doughnut Muffin', MuffinTier.LEGENDARY),
		Muffin('Chocolate Chip Baked Oatmeal Muffin', MuffinTier.LEGENDARY),
		Muffin('Peanut Butter Cadbury Mini Egg Muffin', MuffinTier.LEGENDARY),
		Muffin('Blueberry Cream Cheese Muffin', MuffinTier.LEGENDARY),
		Muffin('Cream Cheese Banana Muffin', MuffinTier.LEGENDARY),
		Muffin('Cream Cheese Filled Chocolate Chip Muffin', MuffinTier.LEGENDARY),
		Muffin('Chocolate Chip Pumpkin Doughnut Muffin', MuffinTier.LEGENDARY),
		Muffin('Pumpkin Apple Streusel Muffin', MuffinTier.LEGENDARY),
		Muffin('Double Chocolate Banana Muffin', MuffinTier.LEGENDARY),
		Muffin('Lemon Pound Cake Muffin', MuffinTier.LEGENDARY),
		Muffin('Pomegranate Cream Cheese Surprise Muffin', MuffinTier.LEGENDARY),
		Muffin('Lemon-Filled Gingerbread Muffin', MuffinTier.LEGENDARY),
		Muffin('Bacon-Peanut Butter Cornbread Muffin', MuffinTier.LEGENDARY),
		Muffin('Coastal Carolina Frittatas Muffin', MuffinTier.LEGENDARY),
		Muffin('Smores Monkey Bread Muffin', MuffinTier.LEGENDARY),
		Muffin('Bakery Style Maple Walnut Muffin', MuffinTier.LEGENDARY),
		Muffin('Cinnamon Sugar Mini-Donut Muffin', MuffinTier.LEGENDARY),
		Muffin('Peanut Butter and Jelly Mini Muffin', MuffinTier.LEGENDARY),
	]

MUFFINS = list()
if not MUFFINS:
	MUFFINS = _create_muffin_list()


def roll_muffin() -> Muffin:
	""" Roll for a new muffin.

	Chooses a random muffin from the list of muffins along the following distribution:
	- 50% Normal
	- 30% Uncommon
	- 15% Rare
	- 5% Legendary
	"""
	roll = random.random()
	selected_tier = None
	if roll > 0.95:
		selected_tier = MuffinTier.LEGENDARY
	elif roll > 0.80:
		selected_tier = MuffinTier.RARE
	elif roll > 0.50:
		selected_tier = MuffinTier.UNCOMMON
	else:
		selected_tier = MuffinTier.NORMAL
	logger.debug(f'Rolled a {str(roll)} which is a ' + str(selected_tier))
	return random.choice([muffin for muffin in MUFFINS if muffin.tier == selected_tier])
